// SPi & ADC

#include "mbed.h"
#include "SPI.h"
#include "DigitalOut.h"
#include "AnalogIn.h"
#include "math.h"

SPI AD9833_SPI(PB_5, PB_4, PB_3);   // PB_5 -> SDATA, 
                                    // PB_4 -> SDATA_Rx (not needed) 
                                    // PB_3 -> SCLK 
DigitalOut fSync(PB_6);             // PB_6 -> FSYNC

AnalogIn   haqqAlla(PC_0);

Serial     pc(USBTX, USBRX); 

// SPI initialization
void spi_init()
{
    // Setup the SPI Setup: 16 bit data, mode 2, with 25MHz clock rate.
    // Note: mBed is MSB first SPI protocol
    AD9833_SPI.format(16, 2);
    AD9833_SPI.frequency(25000000);
    fSync = 1;
    wait_ms(1);
}

// SPI writing
void write_SPI(short dat)
{
    fSync = 0;
    AD9833_SPI.write(dat);
    fSync = 1;
}

// Set desired frequency
void setFreq(long frequency)
{
    int freq_MSB;   // define freq MSB reg value
    int freq_LSB;   // define freq LSB reg value
    
    long freq_cal;  // define freq calculated value
    float freq_val = 0.00000000;    // define ferq calculate tempotary value
    
    //calculate the frqe reg value
    //((desired frequency)/(reference frequency)) x 0x10000000.
    freq_val = (((float)(frequency))/25000000);
    freq_cal = freq_val*0x10000000;
    
    freq_MSB = (int)((freq_cal & 0xFFFC000)>>14); // shift 14 bits
    freq_LSB = (int)(freq_cal & 0x3FFF);
    
    freq_MSB = freq_MSB | 0x4000;  // assign freq reg address
    freq_LSB = freq_LSB | 0x4000;

    write_SPI(0x2100);      // write control reg    - reset
    write_SPI(freq_LSB);    // write freq reg       - LSB
    write_SPI(freq_MSB);    // write freq reg       - MSB
    write_SPI(0xC000);      // write phase reg      - 0 phase shift
    write_SPI(0x2000);      // write control reg    - set
}

// Main function
int main()
{
    long f_init = 10000;
    long f_start = 100000;
    long f_stop = 10000000;
    
    long f_test = 10000000;
    
    float f_vec[256];
    int nsteps = 256;
    
    float ghoxx[256];
    
    spi_init();             // initialized the SPI
    setFreq(f_init);
    wait_ms(10);
    
   
    // create log spaces freq values
    for (int i = 0; i <= 255; i = i + 1) {
        f_vec[i] = (long) ( pow((float) ((float)f_stop/f_start), (float) ((i + 1)/(float)nsteps)))*(f_start);  
    }
    
    // infinite loop
    while(1) {     
    
        // loop through all frequency vlaues
        for (int i = 0; i <= 255; i = i + 1) {    
            setFreq((long) f_vec[i] );
            wait_us(10);
            
            ghoxx[i] = haqqAlla.read();
            wait_us(10);
            ghoxx[i] = ghoxx[i] * 3300;
            pc.printf(" %f \r\n", ghoxx[i]);
  
        } 
            
        for (int i = 0; i <= 255; i++) {
            pc.printf("%f \r\n", ghoxx[i]);            
        }          

    }

}